#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "boundary_writer.hh"

template <class ScalarVector, class Vector>
BoundaryWriter<ScalarVector, Vector>::BoundaryWriter(
    Vector const &vertexCoordinates,
    Dune::BitSetVector<1> const &_boundaryNodes, Projector projector)
    : displacementWriter("displacements", std::fstream::out),
      velocityWriter("velocities", std::fstream::out),
      boundaryNodes(_boundaryNodes),
      projector_(projector) {
  std::fstream vertexCoordinateWriter("coordinates", std::fstream::out);
  for (size_t i = 0; i < boundaryNodes.size(); ++i)
    if (boundaryNodes[i][0])
      vertexCoordinateWriter << vertexCoordinates[i] << std::endl;
  vertexCoordinateWriter.close();
}

template <class ScalarVector, class Vector>
BoundaryWriter<ScalarVector, Vector>::~BoundaryWriter() {
  displacementWriter.close();
  velocityWriter.close();
}

template <class ScalarVector, class Vector>
void BoundaryWriter<ScalarVector, Vector>::writeKinetics(Vector const &u,
                                                         Vector const &v) {
  for (size_t i = 0; i < boundaryNodes.size(); ++i) {
    if (!boundaryNodes[i][0])
      continue;

    displacementWriter << projector_(u[i]) << " ";
    velocityWriter << projector_(v[i]) << " ";
  }

  displacementWriter << std::endl;
  velocityWriter << std::endl;
}

#include "boundary_writer_tmpl.cc"
